@@ -21,17 +21,18 @@ Configuration options registration and useful routines.
 
 import itertools
 import json
-import os
 
 from keystoneauth1 import loading
 from oslo_config import cfg
 from oslo_log import log
 from oslo_middleware import cors
+from oslo_policy import opts
 from osprofiler import opts as profiler
 
 from mistral import version
 
 from mistral._i18n import _
+from mistral.workflow import states
 
 # Options under default group.
 launch_opt = cfg.ListOpt(
@@ -63,6 +64,35 @@ auth_type_opt = cfg.StrOpt(
     help=_('Authentication type (valid options: keystone, keycloak-oidc)')
 )
 
+legacy_action_provider_opts = [
+    cfg.BoolOpt(
+        'load_action_plugins',
+        default=True,
+        help=_(
+            'If True, enables loading actions configured in the '
+            'entry point "mistral.actions".'
+        )
+    ),
+    cfg.BoolOpt(
+        'load_action_generators',
+        default=True,
+        help=_(
+            'If True, enables loading actions from action generators '
+            'configured in the entry point "mistral.generators".'
+        )
+    ),
+    cfg.BoolOpt(
+        'only_builtin_actions',
+        default=False,
+        help=_(
+            'If True, then the legacy action provider loads only '
+            'the actions delivered by the Mistral project out of '
+            'the box plugged in with the entry point "mistral.actions".'
+            'This property is needed mostly for testing.'
+        )
+    ),
+]
+
 api_opts = [
     cfg.HostAddressOpt(
         'host',
@@ -318,7 +348,7 @@ scheduler_opts = [
              'locally before it becomes eligible for processing by other '
              'scheduler instances.'
              'For example, a job needs to run at 12:00:00. When a scheduler '
-             'starts processing it it has 60 seconds (or other configured '
+             'starts processing it has 60 seconds (or other configured '
              'value) to complete the job. If the scheduler did not complete '
              'the job within this period it most likely means that the '
              'scheduler process crashed. In this case another scheduler '
@@ -342,12 +372,12 @@ cron_trigger_opts = [
         default=1,
         min=1,
         help=(
-            'This setting defines how frequently Mistral checks for cron ',
-            'triggers that need execution. By default this is every second ',
-            'which can lead to high system load. Increasing the number will ',
-            'reduce the load but also limit the minimum freqency. For ',
-            'example, a cron trigger can be configured to run every second ',
-            'but if the execution_interval is set to 60, it will only run ',
+            'This setting defines how frequently Mistral checks for cron '
+            'triggers that need execution. By default this is every second '
+            'which can lead to high system load. Increasing the number will '
+            'reduce the load but also limit the minimum freqency. For '
+            'example, a cron trigger can be configured to run every second '
+            'but if the execution_interval is set to 60, it will only run '
             'once per minute.'
         )
     )
@@ -445,6 +475,13 @@ execution_expiration_policy_opts = [
                'The default value is 0. If it is set to 0, '
                'size of batch is total number of expired executions '
                'that is going to be deleted.')
+    ),
+    cfg.ListOpt(
+        'ignored_states',
+        default=[],
+        help='The states that the expiration policy will filter '
+             'out and will not delete.'
+             'Valid values are, [{}]'.format(states.TERMINAL_STATES)
     )
 ]
 
@@ -538,41 +575,18 @@ keycloak_oidc_opts = [
         default='/realms/%s/protocol/openid-connect/userinfo',
         help='Endpoint against which authorization will be performed'
     ),
-]
-
-openstack_actions_opts = [
     cfg.StrOpt(
-        'os-actions-endpoint-type',
-        default=os.environ.get('OS_ACTIONS_ENDPOINT_TYPE', 'public'),
-        choices=['public', 'admin', 'internal'],
-        deprecated_group='DEFAULT',
-        help=_('Type of endpoint in identity service catalog to use for'
-               ' communication with OpenStack services.')
-    ),
-    cfg.ListOpt(
-        'modules-support-region',
-        default=['nova', 'glance', 'heat', 'neutron', 'cinder',
-                 'trove', 'ironic', 'designate', 'murano', 'tacker', 'senlin',
-                 'aodh', 'gnocchi'],
-        help=_('List of module names that support region in actions.')
+        'public_cert_url',
+        default="/realms/%s/protocol/openid-connect/certs",
+        help="URL to get the public key for a particular realm"
     ),
     cfg.StrOpt(
-        'default_region',
-        help=_('Default region name for openstack actions supporting region.')
-    ),
+        'keycloak_iss',
+        help="Keycloak issuer(iss) url. "
+             "Example: https://ip_add:port/auth/realms/%s"
+    )
 ]
 
-# note: this command line option is used only from sync_db and
-# mistral-db-manage
-os_actions_mapping_path = cfg.StrOpt(
-    'openstack_actions_mapping_path',
-    short='m',
-    metavar='MAPPING_PATH',
-    default='actions/openstack/mapping.json',
-    help='Path to openstack action mapping json file.'
-         'It could be relative to mistral package '
-         'directory or absolute.'
-)
 
 yaql_opts = [
     cfg.IntOpt(
@@ -661,6 +675,7 @@ yaql_opts = [
 
 CONF = cfg.CONF
 
+LEGACY_ACTION_PROVIDER_GROUP = 'legacy_action_provider'
 API_GROUP = 'api'
 ENGINE_GROUP = 'engine'
 EXECUTOR_GROUP = 'executor'
@@ -674,7 +689,6 @@ EXECUTION_EXPIRATION_POLICY_GROUP = 'execution_expiration_policy'
 ACTION_HEARTBEAT_GROUP = 'action_heartbeat'
 PROFILER_GROUP = profiler.list_opts()[0][0]
 KEYCLOAK_OIDC_GROUP = "keycloak_oidc"
-OPENSTACK_ACTIONS_GROUP = 'openstack_actions'
 YAQL_GROUP = "yaql"
 KEYSTONE_GROUP = "keystone"
 
@@ -688,6 +702,10 @@ CONF.register_opt(rpc_response_timeout_opt)
 CONF.register_opt(oslo_rpc_executor)
 CONF.register_opt(expiration_token_duration)
 
+CONF.register_opts(
+    legacy_action_provider_opts,
+    group=LEGACY_ACTION_PROVIDER_GROUP
+)
 CONF.register_opts(api_opts, group=API_GROUP)
 CONF.register_opts(engine_opts, group=ENGINE_GROUP)
 CONF.register_opts(executor_opts, group=EXECUTOR_GROUP)
@@ -707,7 +725,6 @@ CONF.register_opts(pecan_opts, group=PECAN_GROUP)
 CONF.register_opts(coordination_opts, group=COORDINATION_GROUP)
 CONF.register_opts(profiler_opts, group=PROFILER_GROUP)
 CONF.register_opts(keycloak_oidc_opts, group=KEYCLOAK_OIDC_GROUP)
-CONF.register_opts(openstack_actions_opts, group=OPENSTACK_ACTIONS_GROUP)
 CONF.register_opts(yaql_opts, group=YAQL_GROUP)
 loading.register_session_conf_options(CONF, KEYSTONE_GROUP)
 
@@ -755,7 +772,6 @@ def list_opts():
         (EXECUTION_EXPIRATION_POLICY_GROUP, execution_expiration_policy_opts),
         (PROFILER_GROUP, profiler_opts),
         (KEYCLOAK_OIDC_GROUP, keycloak_oidc_opts),
-        (OPENSTACK_ACTIONS_GROUP, openstack_actions_opts),
         (YAQL_GROUP, yaql_opts),
         (ACTION_HEARTBEAT_GROUP, action_heartbeat_opts),
         (None, default_group_opts)
@@ -781,6 +797,10 @@ def parse_args(args=None, usage=None, default_config_files=None):
 def set_config_defaults():
     """This method updates all configuration default values."""
     set_cors_middleware_defaults()
+    # TODO(gmann): Remove setting the default value of config policy_file
+    # once oslo_policy change the default value to 'policy.yaml'.
+    # https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
+    opts.set_defaults(CONF, 'policy.yaml')
 
 
 def set_cors_middleware_defaults():
