@@ -1,6 +1,7 @@
 # Copyright 2015 - Mirantis, Inc.
 # Copyright 2015 - StackStorm, Inc.
 # Copyright 2016 - Brocade Communications Systems, Inc.
+# Copyright 2020 Nokia Software.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -20,7 +21,6 @@ import re
 import sys
 import threading
 
-
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_db import sqlalchemy as oslo_sqlalchemy
@@ -41,11 +41,9 @@ from mistral.services import security
 from mistral.workflow import states
 from mistral_lib import utils
 
-
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
-
 _SCHEMA_LOCK = threading.RLock()
 _initialized = False
 
@@ -287,6 +285,13 @@ def _get_collection(model, insecure=False, limit=None, marker=None,
     return query.all()
 
 
+def get_db_objects(model, insecure=False, **filters):
+    query = b.model_query(model) if insecure else _secure_query(model)
+    query = db_filters.apply_filters(query, model, **filters)
+
+    return query
+
+
 def _get_count(model, insecure=False, **filters):
     query = b.model_query(model) if insecure else _secure_query(model)
 
@@ -338,7 +343,7 @@ def _get_db_object_by_name_and_namespace_or_id(model, identifier,
 
 
 def _get_db_object_by_name_and_namespace(model, name,
-                                         namespace, insecure=False,
+                                         namespace='', insecure=False,
                                          columns=()):
     query = (
         b.model_query(model, columns=columns)
@@ -635,6 +640,171 @@ def delete_workflow_definitions(session=None, **kwargs):
     return _delete_all(models.WorkflowDefinition, **kwargs)
 
 
+# Code sources.
+
+@b.session_aware()
+def create_code_source(values, session=None):
+    code_src = models.CodeSource()
+    code_src.update(values.copy())
+
+    try:
+        code_src.save(session=session)
+    except db_exc.DBDuplicateEntry:
+        raise exc.DBDuplicateEntryError(
+            "Duplicate entry for CodeSource ['name', 'namespace',"
+            " 'project_id']: {}, {}, {}".format(code_src.name,
+                                                code_src.namespace,
+                                                code_src.project_id)
+        )
+
+    return code_src
+
+
+@b.session_aware()
+def get_code_sources(fields=None, session=None, **kwargs):
+    return _get_collection(
+        model=models.CodeSource,
+        fields=fields,
+        **kwargs
+    )
+
+
+@b.session_aware()
+def get_code_source(identifier, fields=(), session=None, namespace=''):
+    code_src = _get_db_object_by_name_and_namespace_or_id(
+        models.CodeSource,
+        identifier,
+        namespace=namespace,
+        columns=fields
+    )
+
+    if not code_src:
+        raise exc.DBEntityNotFoundError(
+            "Code Source not found [name=%s,namespace=%s]"
+            % (identifier, namespace)
+        )
+
+    return code_src
+
+
+@b.session_aware()
+def load_code_source(identifier, fields=(), session=None, namespace=''):
+    return _get_db_object_by_name_and_namespace_or_id(
+        models.CodeSource,
+        identifier,
+        namespace=namespace,
+        columns=fields
+    )
+
+
+@b.session_aware()
+def update_code_source(identifier, values, namespace='', session=None):
+    code_src = get_code_source(identifier, namespace=namespace)
+
+    values['version'] = code_src.version + 1
+
+    code_src.update(values.copy())
+
+    return code_src
+
+
+@b.session_aware()
+def delete_code_source(identifier, namespace='', session=None):
+    code_src = get_code_source(
+        identifier,
+        namespace=namespace,
+        session=session
+    )
+
+    session.delete(code_src)
+
+
+@b.session_aware()
+def delete_code_sources(session=None, **kwargs):
+    return _delete_all(models.CodeSource, **kwargs)
+
+
+# Dynamic actions.
+
+@b.session_aware()
+def create_dynamic_action_definition(values, session=None):
+    action_def = models.DynamicActionDefinition()
+
+    action_def.update(values.copy())
+
+    try:
+        action_def.save(session=session)
+    except db_exc.DBDuplicateEntry:
+        raise exc.DBDuplicateEntryError(
+            "Duplicate entry for dynamic action definition "
+            "[name=%s, namespace=%s, project_id=%s]"
+            % (action_def.name, action_def.namespace, action_def.project_id)
+        )
+
+    return action_def
+
+
+@b.session_aware()
+def update_dynamic_action_definition(identifier, values, namespace='',
+                                     session=None):
+    action_def = get_dynamic_action_definition(identifier, namespace=namespace)
+
+    action_def.update(values.copy())
+
+    return action_def
+
+
+@b.session_aware()
+def get_dynamic_action_definition(identifier, fields=(), namespace='',
+                                  session=None):
+    action = _get_db_object_by_name_and_namespace_or_id(
+        models.DynamicActionDefinition,
+        identifier,
+        namespace=namespace,
+        columns=fields
+    )
+
+    if not action:
+        raise exc.DBEntityNotFoundError(
+            "Dynamic Action not found [name=%s,namespace=%s]"
+            % (identifier, namespace)
+        )
+
+    return action
+
+
+@b.session_aware()
+def load_dynamic_action_definition(identifier, fields=(), namespace='',
+                                   session=None):
+    return _get_db_object_by_name_and_namespace_or_id(
+        models.DynamicActionDefinition,
+        identifier,
+        namespace=namespace,
+        columns=fields
+    )
+
+
+@b.session_aware()
+def get_dynamic_action_definitions(fields=None, session=None, **kwargs):
+    return _get_collection(
+        model=models.DynamicActionDefinition,
+        fields=fields,
+        **kwargs
+    )
+
+
+@b.session_aware()
+def delete_dynamic_action_definition(identifier, namespace='', session=None):
+    action_def = get_dynamic_action_definition(identifier, namespace)
+
+    session.delete(action_def)
+
+
+@b.session_aware()
+def delete_dynamic_action_definitions(session=None, **kwargs):
+    return _delete_all(models.DynamicActionDefinition, **kwargs)
+
+
 # Action definitions.
 
 @b.session_aware()
@@ -654,26 +824,39 @@ def get_action_definition_by_id(id, fields=(), session=None):
 
 
 @b.session_aware()
-def get_action_definition(identifier, fields=(), session=None):
+def get_action_definition(identifier, fields=(), session=None, namespace=''):
     a_def = _get_db_object_by_name_and_namespace_or_id(
         models.ActionDefinition,
         identifier,
+        namespace=namespace,
         columns=fields
     )
 
+    #  If the action was not found in the given namespace,
+    #  look in the default namespace
+    if not a_def:
+        a_def = _get_db_object_by_name_and_namespace_or_id(
+            models.ActionDefinition,
+            identifier,
+            namespace='',
+            columns=fields
+        )
+
     if not a_def:
         raise exc.DBEntityNotFoundError(
-            "Action definition not found [action_name=%s]" % identifier
+            "Action definition not found [action_name=%s, namespace=%s]"
+            % (identifier, namespace)
         )
 
     return a_def
 
 
 @b.session_aware()
-def load_action_definition(name, fields=(), session=None):
-    return _get_db_object_by_name(
+def load_action_definition(name, fields=(), session=None, namespace=''):
+    return _get_db_object_by_name_and_namespace(
         models.ActionDefinition,
         name,
+        namespace=namespace,
         columns=fields
     )
 
@@ -693,8 +876,8 @@ def create_action_definition(values, session=None):
         a_def.save(session=session)
     except db_exc.DBDuplicateEntry:
         raise exc.DBDuplicateEntryError(
-            "Duplicate entry for Action ['name', 'project_id']:"
-            " {}, {}".format(a_def.name, a_def.project_id)
+            "Duplicate entry for Action ['name', 'namespace', 'project_id']:"
+            " {}, {}, {}".format(a_def.name, a_def.namespace, a_def.project_id)
         )
 
     return a_def
@@ -702,7 +885,8 @@ def create_action_definition(values, session=None):
 
 @b.session_aware()
 def update_action_definition(identifier, values, session=None):
-    a_def = get_action_definition(identifier)
+    namespace = values.get('namespace', '')
+    a_def = get_action_definition(identifier, namespace=namespace)
 
     a_def.update(values.copy())
 
@@ -711,15 +895,19 @@ def update_action_definition(identifier, values, session=None):
 
 @b.session_aware()
 def create_or_update_action_definition(name, values, session=None):
-    if not _get_db_object_by_name(models.ActionDefinition, name):
+    namespace = values.get('namespace', '')
+    if not _get_db_object_by_name_and_namespace(
+            models.ActionDefinition,
+            name,
+            namespace=namespace):
         return create_action_definition(values)
     else:
         return update_action_definition(name, values)
 
 
 @b.session_aware()
-def delete_action_definition(identifier, session=None):
-    a_def = get_action_definition(identifier)
+def delete_action_definition(identifier, namespace='', session=None):
+    a_def = get_action_definition(identifier, namespace=namespace)
 
     session.delete(a_def)
 
@@ -793,8 +981,8 @@ def update_action_execution_heartbeat(id, session=None):
         raise exc.DBEntityNotFoundError
 
     now = utils.utc_now_sec()
-    session.query(models.ActionExecution).\
-        filter(models.ActionExecution.id == id).\
+    session.query(models.ActionExecution). \
+        filter(models.ActionExecution.id == id). \
         update({'last_heartbeat': now})
 
 
@@ -901,7 +1089,7 @@ def delete_workflow_execution(id, session=None):
             raise exc.DBEntityNotFoundError(
                 "WorkflowExecution not found [id=%s]" % id
             )
-    except db_exc.DBError as e:
+    except Exception as e:
         if is_mysql_max_depth_error(e) or is_mariadb_max_depth_error(e):
             # https://bugs.launchpad.net/mistral/+bug/1832300
             # mysql cascade delete error
@@ -1393,17 +1581,18 @@ def get_superfluous_executions(max_finished_executions, limit=None, columns=(),
 def _get_completed_root_executions_query(columns):
     query = b.model_query(models.WorkflowExecution, columns=columns)
 
-    # Only WorkflowExecution that are not a child of other WorkflowExecution.
+    # This is an empty list by default.
+    ignored_states = CONF.execution_expiration_policy.ignored_states
+    desired_states = states.TERMINAL_STATES - set(ignored_states)
+
+    # Only workflow executions that are not a child of
+    # other workflow executions.
     query = query.filter(
         models.WorkflowExecution.task_execution_id == sa.null()
     )
 
     query = query.filter(
-        models.WorkflowExecution.state.in_(
-            [states.SUCCESS,
-             states.ERROR,
-             states.CANCELLED]
-        )
+        models.WorkflowExecution.state.in_(desired_states)
     )
 
     return query
@@ -1757,8 +1946,8 @@ def update_resource_member(resource_id, res_type, member_id, values,
 
 @b.session_aware()
 def delete_resource_member(resource_id, res_type, member_id, session=None):
-    query = _secure_query(models.ResourceMember).\
-        filter_by(resource_type=res_type).\
+    query = _secure_query(models.ResourceMember). \
+        filter_by(resource_type=res_type). \
         filter(_get_criterion(resource_id, member_id))
 
     # TODO(kong): Check association with cron triggers when deleting a workflow
