@@ -77,27 +77,50 @@ class EngineServer(service_base.MistralService):
         # in the middle of executing an action then one of the remaining
         # engine instances will expire the action in a configured period
         # of time.
-        if cfg.CONF.executor.type == 'local':
+        if CONF.executor.type == 'local':
             action_heartbeat_sender.start()
 
         if self._setup_profiler:
-            profiler_utils.setup('mistral-engine', cfg.CONF.engine.host)
+            profiler_utils.setup('mistral-engine', CONF.engine.host)
 
         # Initialize and start RPC server.
 
-        self._rpc_server = rpc.get_rpc_server_driver()(cfg.CONF.engine)
+        self._rpc_server = rpc.get_rpc_server_driver()(CONF.engine)
         self._rpc_server.register_endpoint(self)
 
-        self._rpc_server.run(executor=cfg.CONF.oslo_rpc_executor)
+        self._rpc_server.run(executor=CONF.oslo_rpc_executor)
 
         self._notify_started('Engine server started.')
 
     def stop(self, graceful=False):
+        # NOTE(rakhmerov): Unfortunately, oslo.service doesn't pass the
+        # 'graceful' parameter with a correct value. It's simply ignored
+        # in the corresponding call chain leading to a concrete service.
+        # The only workaround for now is to check 'graceful_shutdown_timeout'
+        # configuration option. If it's not empty (not None or 0) then we
+        # should treat it a graceful shutdown.
+        graceful = bool(CONF.graceful_shutdown_timeout)
+
+        LOG.info(
+            'Stopping an engine server [graceful=%s, timeout=%s]',
+            graceful,
+            CONF.graceful_shutdown_timeout
+        )
+
         super(EngineServer, self).stop(graceful)
 
+        # The rpc server needs to be stopped first so that the engine
+        # server stops receiving new RPC calls. Under load, this operation
+        # may take much time in case of graceful shutdown because there
+        # still may be RPC messages already polled from the queue and
+        # waiting for processing. So an underlying RPC server has to wait
+        # until they are processed.
+        if self._rpc_server:
+            self._rpc_server.stop(graceful)
+
         action_heartbeat_checker.stop(graceful)
 
-        if cfg.CONF.executor.type == 'local':
+        if CONF.executor.type == 'local':
             action_heartbeat_sender.stop(graceful)
 
         if self._scheduler:
@@ -108,8 +131,8 @@ class EngineServer(service_base.MistralService):
         if self._expiration_policy_tg:
             self._expiration_policy_tg.stop(graceful)
 
-        if self._rpc_server:
-            self._rpc_server.stop(graceful)
+    def wait(self):
+        LOG.info("Waiting for an engine server to exit...")
 
     def start_workflow(self, rpc_ctx, wf_identifier, wf_namespace,
                        wf_ex_id, wf_input, description, params):
@@ -145,22 +168,24 @@ class EngineServer(service_base.MistralService):
         )
 
     def start_action(self, rpc_ctx, action_name,
-                     action_input, description, params):
+                     action_input, description, namespace, params):
         """Receives calls over RPC to start actions on engine.
 
         :param rpc_ctx: RPC request context.
         :param action_name: name of the Action.
         :param action_input: input dictionary for Action.
         :param description: description of new Action execution.
+        :param namespace: The namespace of the action.
         :param params: extra parameters to run Action.
         :return: Action execution.
         """
         LOG.info(
             "Received RPC request 'start_action'[name=%s, input=%s, "
-            "description=%s, params=%s]",
+            "description=%s, namespace=%s params=%s]",
             action_name,
             utils.cut(action_input),
             description,
+            namespace,
             params
         )
 
@@ -168,6 +193,7 @@ class EngineServer(service_base.MistralService):
             action_name,
             action_input,
             description,
+            namespace=namespace,
             **params
         )
 
@@ -186,7 +212,6 @@ class EngineServer(service_base.MistralService):
             action_ex_id,
             result.cut_repr() if result else '<unknown>'
         )
-
         return self.engine.on_action_complete(action_ex_id, result, wf_action)
 
     def on_action_update(self, rpc_ctx, action_ex_id, state, wf_action):
