@@ -1,6 +1,7 @@
 # Copyright 2016 - Nokia Networks.
 # Copyright 2016 - Brocade Communications Systems, Inc.
 # Copyright 2018 - Extreme Networks, Inc.
+# Copyright 2019 - NetCracker Technology Corp.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -15,13 +16,12 @@
 #    limitations under the License.
 
 import abc
-import collections
+from collections import abc as collections_abc
 import copy
 import json
 from oslo_config import cfg
 from oslo_log import log as logging
 from osprofiler import profiler
-import six
 
 from mistral.db.v2 import api as db_api
 from mistral.engine import actions
@@ -33,6 +33,7 @@ from mistral import exceptions as exc
 from mistral import expressions as expr
 from mistral.notifiers import base as notif
 from mistral.notifiers import notification_events as events
+from mistral.services import actions as action_service
 from mistral.utils import wf_trace
 from mistral.workflow import base as wf_base
 from mistral.workflow import commands
@@ -40,12 +41,10 @@ from mistral.workflow import data_flow
 from mistral.workflow import states
 from mistral_lib import utils
 
-
 LOG = logging.getLogger(__name__)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class Task(object):
+class Task(object, metaclass=abc.ABCMeta):
     """Task.
 
     Represents a workflow task and defines interface that can be used by
@@ -68,16 +67,18 @@ class Task(object):
         self.created = False
         self.state_changed = False
 
-    def notify(self, old_task_state, new_task_state):
+    def _notify(self, from_state, to_state):
         publishers = self.wf_ex.params.get('notify')
 
         if not publishers and not isinstance(publishers, list):
             return
 
         notifier = notif.get_notifier(cfg.CONF.notifier.type)
-        event = events.identify_task_event(old_task_state, new_task_state)
+
+        event = events.identify_task_event(from_state, to_state)
 
         filtered_publishers = []
+
         for publisher in publishers:
             if not isinstance(publisher, dict):
                 continue
@@ -90,30 +91,46 @@ class Task(object):
         if not filtered_publishers:
             return
 
-        def _convert_to_notification_data():
-            return {
-                "id": self.task_ex.id,
-                "name": self.task_ex.name,
-                "workflow_execution_id": self.task_ex.workflow_execution_id,
-                "workflow_name": self.task_ex.workflow_name,
-                "workflow_namespace": self.task_ex.workflow_namespace,
-                "workflow_id": self.task_ex.workflow_id,
-                "state": self.task_ex.state,
-                "state_info": self.task_ex.state_info,
-                "type": self.task_ex.type,
-                "project_id": self.task_ex.project_id
-            }
+        data = {
+            "id": self.task_ex.id,
+            "name": self.task_ex.name,
+            "workflow_execution_id": self.task_ex.workflow_execution_id,
+            "workflow_name": self.task_ex.workflow_name,
+            "workflow_namespace": self.task_ex.workflow_namespace,
+            "workflow_id": self.task_ex.workflow_id,
+            "state": self.task_ex.state,
+            "state_info": self.task_ex.state_info,
+            "type": self.task_ex.type,
+            "project_id": self.task_ex.project_id,
+            "created_at": utils.datetime_to_str(self.task_ex.created_at),
+            "updated_at": utils.datetime_to_str(self.task_ex.updated_at),
+            "started_at": utils.datetime_to_str(self.task_ex.started_at),
+            "finished_at": utils.datetime_to_str(self.task_ex.finished_at)
+        }
 
         def _send_notification():
             notifier.notify(
                 self.task_ex.id,
-                _convert_to_notification_data(),
+                data,
                 event,
                 self.task_ex.updated_at,
                 filtered_publishers
             )
+
         post_tx_queue.register_operation(_send_notification)
 
+    def get_id(self):
+        return self.task_ex.id if self.task_ex else None
+
+    def get_name(self):
+        return self.task_ex.name if self.task_ex else None
+
+    def get_state(self):
+        return self.task_ex.state if self.task_ex else None
+
+    def get_state_info(self):
+        return self.task_ex.state_info if self.task_ex else None
+
     def is_completed(self):
         return self.task_ex and states.is_completed(self.task_ex.state)
 
@@ -126,6 +143,85 @@ class Task(object):
     def is_state_changed(self):
         return self.state_changed
 
+    def get_expression_context(self, ctx=None):
+        assert self.task_ex
+
+        return data_flow.ContextView(
+            data_flow.get_current_task_dict(self.task_ex),
+            data_flow.get_workflow_environment_dict(self.wf_ex),
+            ctx or {},
+            self.task_ex.in_context,
+            self.wf_ex.context,
+            self.wf_ex.input,
+        )
+
+    def evaluate(self, data, ctx=None):
+        """Evaluates data against the task context.
+
+        The data is evaluated against the standard task context that includes
+        all standard components like workflow environment, workflow input etc.
+        However, if needed, the method also takes an additional context that
+        can be virtually merged with the standard one.
+
+        :param data: Data (a string, dict or a list) that possibly contain
+            YAQL/Jinja expressions to evaluate.
+        :param ctx: Additional context.
+        :return:
+        """
+
+        return expr.evaluate_recursively(
+            data,
+            self.get_expression_context(ctx)
+        )
+
+    def set_runtime_context_value(self, key, value):
+        assert self.task_ex
+
+        if not self.task_ex.runtime_context:
+            self.task_ex.runtime_context = {}
+
+        self.task_ex.runtime_context[key] = value
+
+    def touch_runtime_context(self):
+        """Must be called after any update of the runtime context.
+
+
+        The ORM framework can't trace updates happening within
+        deep structures like dictionaries. So every time we update
+        something inside "runtime_context" of the task execution
+        we need to call this method that does a fake update of the
+        field.
+        """
+        runtime_ctx = self.task_ex.runtime_context
+
+        if runtime_ctx:
+            random_key = next(iter(runtime_ctx.keys()))
+            runtime_ctx[random_key] = runtime_ctx[random_key]
+
+    def cleanup_runtime_context(self):
+        runtime_context = self.task_ex.runtime_context
+
+        if runtime_context:
+            runtime_context.clear()
+
+    def get_policy_context(self, key):
+        assert self.task_ex
+
+        if not self.task_ex.runtime_context:
+            self.task_ex.runtime_context = {}
+
+        if key not in self.task_ex.runtime_context:
+            self.task_ex.runtime_context.update({key: {}})
+
+        return self.task_ex.runtime_context[key]
+
+    def invalidate_result(self):
+        if not self.task_ex:
+            return
+
+        for ex in self.task_ex.executions:
+            ex.accepted = False
+
     @abc.abstractmethod
     def on_action_complete(self, action_ex):
         """Handle action completion.
@@ -207,12 +303,6 @@ class Task(object):
 
         cur_state = self.task_ex.state
 
-        # Set initial started_at in case of waiting => running.
-        # We can't set this just in run_existing, because task retries
-        # will update started_at, which is incorrect.
-        if cur_state == states.WAITING and state == states.RUNNING:
-            self.save_started_time()
-
         if cur_state != state or self.task_ex.state_info != state_info:
             task_ex = db_api.update_task_execution_state(
                 id=self.task_ex.id,
@@ -229,9 +319,25 @@ class Task(object):
                 if isinstance(state_info, dict) else state_info
             self.state_changed = True
 
+            # Recalculating "started_at" timestamp only if the state
+            # was WAITING (all preconditions are satisfied and it's
+            # ready to start) or IDLE, or the task is being rerun. So
+            # we treat all iterations of "retry" policy as one run.
+            if state == states.RUNNING and \
+                    (cur_state in (None, states.WAITING) or self.rerun):
+                self.task_ex.started_at = utils.utc_now_sec()
+
+            if states.is_completed(state):
+                self.task_ex.finished_at = utils.utc_now_sec()
+
+            if self.rerun:
+                self.task_ex.finished_at = None
+
             if processed is not None:
                 self.task_ex.processed = processed
 
+            self._notify(cur_state, state)
+
             wf_trace.info(
                 self.task_ex.workflow_execution,
                 "Task '%s' (%s) [%s -> %s, msg=%s]" %
@@ -258,15 +364,8 @@ class Task(object):
 
         assert self.task_ex
 
-        # Record the current task state.
-        old_task_state = self.task_ex.state
-
         # Ignore if task already completed.
         if self.is_completed():
-            # Publish task event again so subscribers know
-            # task completed state is being processed again.
-            self.notify(old_task_state, self.task_ex.state)
-
             return
 
         # If we were unable to change the task state it means that it was
@@ -316,9 +415,6 @@ class Task(object):
         # If workflow is paused we shouldn't schedule new commands
         # and mark task as processed.
         if states.is_paused(self.wf_ex.state):
-            # Publish task event even if the workflow is paused.
-            self.notify(old_task_state, self.task_ex.state)
-
             return
 
         # Mark task as processed after all decisions have been made
@@ -327,11 +423,6 @@ class Task(object):
 
         self.register_workflow_completion_check()
 
-        self.save_finished_time()
-
-        # Publish task event.
-        self.notify(old_task_state, self.task_ex.state)
-
         dispatcher.dispatch_workflow_commands(self.wf_ex, cmds)
 
     def register_workflow_completion_check(self):
@@ -358,15 +449,8 @@ class Task(object):
 
         assert self.task_ex
 
-        # Record the current task state.
-        old_task_state = self.task_ex.state
-
         # Ignore if task already completed.
         if states.is_completed(self.task_ex.state):
-            # Publish task event again so subscribers know
-            # task completed state is being processed again.
-            self.notify(old_task_state, self.task_ex.state)
-
             return
 
         # Update only if state transition is valid.
@@ -385,34 +469,28 @@ class Task(object):
         if states.is_completed(self.task_ex.state):
             self.register_workflow_completion_check()
 
-        # Publish event.
-        self.notify(old_task_state, self.task_ex.state)
-
     def _before_task_start(self):
         policies_spec = self.task_spec.get_policies()
 
         for p in policies.build_policies(policies_spec, self.wf_spec):
-            p.before_task_start(self.task_ex, self.task_spec)
+            p.before_task_start(self)
 
     def _after_task_complete(self):
         policies_spec = self.task_spec.get_policies()
 
         for p in policies.build_policies(policies_spec, self.wf_spec):
-            p.after_task_complete(self.task_ex, self.task_spec)
+            p.after_task_complete(self)
 
     @profiler.trace('task-create-task-execution')
-    def _create_task_execution(self, state=states.RUNNING, state_info=None):
-        task_id = utils.generate_unicode_uuid()
-        task_name = self.task_spec.get_name()
-        task_type = self.task_spec.get_type()
-
+    def _create_task_execution(self, state=None, state_info=None):
         values = {
-            'id': task_id,
-            'name': task_name,
+            'id': utils.generate_unicode_uuid(),
+            'name': self.task_spec.get_name(),
             'workflow_execution_id': self.wf_ex.id,
             'workflow_name': self.wf_ex.workflow_name,
             'workflow_namespace': self.wf_ex.workflow_namespace,
             'workflow_id': self.wf_ex.workflow_id,
+            'tags': self.task_spec.get_tags(),
             'state': state,
             'state_info': state_info,
             'spec': self.task_spec.to_dict(),
@@ -421,7 +499,7 @@ class Task(object):
             'published': {},
             'runtime_context': {},
             'project_id': self.wf_ex.project_id,
-            'type': task_type
+            'type': self.task_spec.get_type(),
         }
 
         if self.triggered_by:
@@ -457,18 +535,6 @@ class Task(object):
 
         return env.get('__actions', {}).get(action_name, {})
 
-    def save_started_time(self, value='default'):
-        if not self.task_ex:
-            return
-        time = value if value is not 'default' else utils.utc_now_sec()
-        self.task_ex.started_at = time
-
-    def save_finished_time(self, value='default'):
-        if not self.task_ex:
-            return
-        time = value if value is not 'default' else utils.utc_now_sec()
-        self.task_ex.finished_at = time
-
 
 class RegularTask(Task):
     """Regular task.
@@ -511,10 +577,10 @@ class RegularTask(Task):
             return
 
         self._create_task_execution()
-        self.save_started_time()
 
-        # Publish event.
-        self.notify(None, self.task_ex.state)
+        # Set the initial state and trigger all operations
+        # related to the state change.
+        self.set_state(states.RUNNING, None)
 
         LOG.debug(
             'Starting task [name=%s, init_state=%s, workflow_name=%s,'
@@ -545,17 +611,9 @@ class RegularTask(Task):
                 'Rerunning succeeded tasks is not supported.'
             )
 
-        # Record the current task state.
-        old_task_state = self.task_ex.state
-
         self.set_state(states.RUNNING, None, processed=False)
 
-        # Publish event.
-        self.notify(old_task_state, self.task_ex.state)
-
         if self.rerun:
-            self.save_started_time()
-            self.save_finished_time(value=None)
             self._before_task_start()
 
             # Policies could possibly change task state.
@@ -609,8 +667,6 @@ class RegularTask(Task):
 
         action = self._build_action()
 
-        action.validate_input(input_dict)
-
         action.schedule(
             input_dict,
             target,
@@ -641,7 +697,7 @@ class RegularTask(Task):
         input_spec = self.task_spec.get_input()
 
         input_dict = (
-            self._evaluate_expression(input_spec, ctx) if input_spec else {}
+            self.evaluate(input_spec, ctx) if input_spec else {}
         )
 
         if not isinstance(input_dict, dict):
@@ -657,48 +713,66 @@ class RegularTask(Task):
             overwrite=False
         )
 
-    def _evaluate_expression(self, expression, ctx=None):
-        ctx_view = data_flow.ContextView(
-            data_flow.get_current_task_dict(self.task_ex),
-            data_flow.get_workflow_environment_dict(self.wf_ex),
-            ctx or self.ctx,
-            self.wf_ex.context,
-            self.wf_ex.input,
-            self.task_ex.in_context,
-        )
+    def _get_action_name(self):
+        result = self.task_spec.get_action_name()
 
-        return expr.evaluate_recursively(expression, ctx_view)
+        # An action name can be an expression so we reevaluate it.
+        if result:
+            result = self.evaluate(result)
+
+        if not result:
+            result = 'std.noop'
+
+        return result
 
     def _build_action(self):
-        action_name = self.task_spec.get_action_name()
         wf_name = self.task_spec.get_workflow_name()
 
-        # For dynamic workflow evaluation we regenerate the action.
+        # A workflow name can be an expression so we reevaluate it.
         if wf_name:
             return actions.WorkflowAction(
-                wf_name=self._evaluate_expression(wf_name),
+                wf_name=self.evaluate(wf_name),
                 task_ex=self.task_ex
             )
 
-        # For dynamic action evaluation we just regenerate the name.
-        if action_name:
-            action_name = self._evaluate_expression(action_name)
+        action_desc = self._get_action_descriptor()
 
-        if not action_name:
-            action_name = 'std.noop'
-
-        action_def = actions.resolve_action_definition(
-            action_name,
-            self.wf_ex.name,
-            self.wf_spec.get_name()
+        return actions.RegularAction(
+            action_desc=action_desc,
+            task_ex=self.task_ex,
+            task_ctx=self.ctx
         )
 
-        if action_def.spec:
-            return actions.AdHocAction(action_def, task_ex=self.task_ex,
-                                       task_ctx=self.ctx,
-                                       wf_ctx=self.wf_ex.context)
+    def _get_action_descriptor(self):
+        res = None
 
-        return actions.PythonAction(action_def, task_ex=self.task_ex)
+        action_name = self._get_action_name()
+        namespace = self.wf_ex.workflow_namespace
+
+        provider = action_service.get_system_action_provider()
+
+        wb_name = self.wf_ex.runtime_context.get('wb_name')
+
+        if wb_name:
+            # First try to find the action within the same workbook
+            # that the workflow belongs to.
+            res = provider.find(
+                '%s.%s' % (wb_name, action_name),
+                namespace=namespace
+            )
+
+        if res is None:
+            # Now try to find by the action name as it appears in the
+            # workflow text.
+            res = provider.find(action_name, namespace=namespace)
+
+        if res is None:
+            raise exc.MistralException(
+                "Failed to find action [action_name=%s, namespace=%s]"
+                % (action_name, namespace)
+            )
+
+        return res
 
     def _get_timeout(self):
         timeout = self.task_spec.get_policies().get_timeout()
@@ -778,7 +852,7 @@ class WithItemsTask(RegularTask):
         with_items_values = self._get_with_items_values()
 
         if self._is_new():
-            action_count = len(six.next(iter(with_items_values.values())))
+            action_count = len(next(iter(with_items_values.values())))
 
             self._prepare_runtime_context(action_count)
 
@@ -794,8 +868,6 @@ class WithItemsTask(RegularTask):
 
             action = self._build_action()
 
-            action.validate_input(input_dict)
-
             action.schedule(
                 input_dict,
                 target,
@@ -826,7 +898,7 @@ class WithItemsTask(RegularTask):
         :return: Evaluated 'with-items' expression values.
         """
 
-        exp_res = self._evaluate_expression(self.task_spec.get_with_items())
+        exp_res = self.evaluate(self.task_spec.get_with_items())
 
         # Expression result may contain iterables instead of lists in the
         # dictionary values. So we need to convert them into lists and
@@ -837,7 +909,7 @@ class WithItemsTask(RegularTask):
         required_len = -1
 
         for var, items in exp_res.items():
-            if not isinstance(items, collections.Iterable):
+            if not isinstance(items, collections_abc.Iterable):
                 raise exc.InputException(
                     "Wrong input format for: %s. Iterable type is"
                     " expected for each value." % result
@@ -967,10 +1039,11 @@ class WithItemsTask(RegularTask):
             indices = copy.copy(candidates)
 
             if max(candidates) < count - 1:
-                indices += list(six.moves.range(max(candidates) + 1, count))
+                indices += list(range(max(candidates) + 1, count))
         else:
             i = self._get_next_start_index()
-            indices = list(six.moves.range(i, count))
+
+            indices = list(range(i, count))
 
         return indices[:capacity]
 
