@@ -14,10 +14,12 @@
 #    limitations under the License.
 
 import abc
+from functools import wraps
 
 from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging as messaging
+from oslo_messaging.exceptions import MessagingTimeout
 from oslo_messaging.rpc import client
 from stevedore import driver
 
@@ -85,25 +87,38 @@ def _wrap_exception_and_reraise(exception):
 
 
 def wrap_messaging_exception(method):
-    """This decorator unwrap remote error in one of MistralException.
+    """The decorator unwraps a remote error into one of the mistral exceptions.
 
-    oslo.messaging has different behavior on raising exceptions
-    when fake or rabbit transports are used. In case of rabbit
-    transport it raises wrapped RemoteError which forwards directly
-    to API. Wrapped RemoteError contains one of MistralException raised
-    remotely on Engine and for correct exception interpretation we
-    need to unwrap and raise given exception and manually send it to
-    API layer.
+    oslo.messaging has different behavior on raising exceptions depending on
+    whether we use 'fake' or 'rabbit' transports. In case of 'rabbit' transport
+    it raises an instance of RemoteError which forwards directly to the API.
+    The RemoteError instance contains one of the MistralException instances
+    raised remotely on the RPC server side and for correct exception handling
+    we need to unwrap and raise the original wrapped exception.
     """
+
+    @wraps(method)
     def decorator(*args, **kwargs):
         try:
             return method(*args, **kwargs)
-
         except exc.MistralException:
             raise
+        except MessagingTimeout:
+            timeout = cfg.CONF.rpc_response_timeout
+            raise exc.MistralException('This rpc call "%s" took longer than '
+                                       'configured %s seconds.' %
+                                       (method.__name__, timeout))
         except (client.RemoteError, exc.KombuException, Exception) as e:
+            # Since we're going to transform the original exception
+            # we need to log it as is.
+            LOG.exception(
+                "Caught a messaging remote error."
+                " See details of the original exception."
+            )
+
             if hasattr(e, 'exc_type') and hasattr(exc, e.exc_type):
                 exc_cls = getattr(exc, e.exc_type)
+
                 raise exc_cls(e.value)
 
             _wrap_exception_and_reraise(e)
@@ -131,7 +146,7 @@ class RPCClient(object):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def async_call(self, ctx, method, target=None, **kwargs):
+    def async_call(self, ctx, method, target=None, fanout=False, **kwargs):
         """Asynchronous call of RPC method.
 
         Does not block the thread, just send invoking data to
@@ -162,7 +177,7 @@ class RPCServer(object):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def run(self, executor='blocking'):
+    def run(self, executor='eventlet'):
         """Runs the RPC server.
 
         :param executor: Executor used to process incoming requests. Different
