@@ -15,7 +15,6 @@
 import croniter
 import datetime
 import json
-import six
 
 from oslo_log import log as logging
 
@@ -77,7 +76,7 @@ def create_cron_trigger(name, workflow_name, workflow_input,
     if not start_time:
         start_time = datetime.datetime.utcnow()
 
-    if isinstance(first_time, six.string_types):
+    if isinstance(first_time, str):
         try:
             first_time = datetime.datetime.strptime(
                 first_time,
@@ -221,6 +220,8 @@ def delete_event_trigger(event_trigger):
         list(events)
     )
 
+    security.delete_trust(event_trigger['trust_id'])
+
 
 def update_event_trigger(id, values):
     trig = db_api.update_event_trigger(id, values)
@@ -236,6 +237,9 @@ def on_workflow_complete(wf_ex):
     if wf_ex.task_execution_id:
         return
 
+    if not wf_ex.description:
+        return
+
     try:
         description = json.loads(wf_ex.description)
     except ValueError as e:
