@@ -15,8 +15,8 @@
 
 import copy
 import time
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 
 from mistral import context as auth_context
@@ -104,7 +104,7 @@ class EventEngineTest(base.DbTestCase):
         auth_context.set_ctx(self.ctx)
 
         t['scope'] = 'public'
-        t['project_id'] = self.ctx.tenant
+        t['project_id'] = self.ctx.project_id
         trigger = db_api.create_event_trigger(t)
 
         # Switch to the user.
@@ -140,7 +140,10 @@ class EventEngineTest(base.DbTestCase):
 
             args, kwargs = client_mock.start_workflow.call_args
 
-            self.assertEqual((EVENT_TRIGGER['workflow_id'], '', {}), args)
+            self.assertEqual(
+                (EVENT_TRIGGER['workflow_id'], '', None, {}),
+                args
+            )
             self.assertDictEqual(
                 {
                     'service': 'fake_publisher',
@@ -181,7 +184,10 @@ class EventEngineTest(base.DbTestCase):
 
             args, kwargs = client_mock.start_workflow.call_args
 
-            self.assertEqual((EVENT_TRIGGER['workflow_id'], '', {}), args)
+            self.assertEqual(
+                (EVENT_TRIGGER['workflow_id'], '', None, {}),
+                args
+            )
             self.assertDictEqual(
                 {
                     'service': 'fake_publisher',
