@@ -15,13 +15,13 @@
 
 from oslo_config import cfg
 from oslo_log import log as logging
+from osprofiler import profiler
 
 from mistral import context as auth_ctx
 from mistral.db.v2.sqlalchemy import models
 from mistral import exceptions as exc
 from mistral import expressions as expr
 from mistral.lang import parser as spec_parser
-from mistral.utils import inspect_utils
 from mistral.workflow import states
 from mistral_lib import utils
 
@@ -47,7 +47,7 @@ class ContextView(dict):
     def __init__(self, *dicts):
         super(ContextView, self).__init__()
 
-        self.dicts = dicts or []
+        self.dicts = [d for d in dicts if d is not None]
 
     def __getitem__(self, key):
         for d in self.dicts:
@@ -156,16 +156,10 @@ def _extract_execution_result(ex):
         return ex.output['result']
 
 
-def invalidate_task_execution_result(task_ex):
-    for ex in task_ex.executions:
-        ex.accepted = False
-
-
 def get_task_execution_result(task_ex):
     execs = task_ex.executions
-    execs.sort(
-        key=lambda x: x.runtime_context.get('index')
-    )
+
+    execs.sort(key=lambda x: x.runtime_context.get('index'))
 
     results = [
         _extract_execution_result(ex)
@@ -173,16 +167,9 @@ def get_task_execution_result(task_ex):
         if hasattr(ex, 'output') and ex.accepted
     ]
 
-    task_spec = spec_parser.get_task_spec(task_ex.spec)
-
-    if task_spec.get_with_items():
-        # TODO(rakhmerov): Smell: violation of 'with-items' encapsulation.
-        with_items_ctx = task_ex.runtime_context.get('with_items')
-
-        if with_items_ctx and with_items_ctx.get('count') > 0:
-            return results
-        else:
-            return []
+    # If it's a 'with-items' task we should always return an array.
+    if spec_parser.get_task_spec(task_ex.spec).get_with_items():
+        return results
 
     return results[0] if len(results) == 1 else results
 
@@ -233,6 +220,10 @@ def publish_variables(task_ex, task_spec):
     #    {'branch': {vars}, 'global': {vars}, 'atomic': {vars}}
 
 
+@profiler.trace(
+    'data-flow-evaluate-task-outbound-context',
+    hide_args=True
+)
 def evaluate_task_outbound_context(task_ex):
     """Evaluates task outbound Data Flow context.
 
@@ -322,15 +313,6 @@ def add_workflow_variables_to_context(wf_ex, wf_spec):
     utils.merge_dicts(wf_ex.context, wf_vars)
 
 
-def evaluate_object_fields(obj, context):
-    fields = inspect_utils.get_public_fields(obj)
-
-    evaluated_fields = expr.evaluate_recursively(fields, context)
-
-    for k, v in evaluated_fields.items():
-        setattr(obj, k, v)
-
-
 def get_workflow_environment_dict(wf_ex):
     if not wf_ex:
         return {}
